import { NextResponse, NextRequest } from 'next/server'
import { validateToken, decryptToken } from '@/lib/jwt'
import { db } from "@/lib/db"
 
interface orCardFilterProps {
  sanitized_name:    string
  OR:         orSetFilterProps[]
} 

interface orSetFilterProps {
  set_code:          string
}

interface cardEntryAPIProps {
  amount: number,
  sanitized_name: string,
  set: string
}

export async function POST(req: NextRequest) {
  try {
    const token = req?.headers.get("authorization")?.split(" ")[1]
    const { name, cards, selected_bset, commander_name } = await req.json()

    if(token == undefined) {
      return NextResponse.json({"message": "You did not provide a token."},{
        status: 401,
      });
    }
    
    if(!validateToken(token)) {
      return NextResponse.json({"message": "Your token is not valid."},{
        status: 401,
      });
    }

    const tokenData = decryptToken(token)

    if(name == undefined || cards == undefined || selected_bset == undefined || commander_name == undefined) {
      return NextResponse.json({"message": "Wrong data in the request."},{
        status: 401,
      });
    }

    const bset = await db.bset.findFirst({
      where: {
        sanitized_name: selected_bset
      },
      relationLoadStrategy: "join",
      include: {
        sets: true
      }
    })

    const set_codes: orSetFilterProps[] = []
    bset?.sets.forEach((set) => {
      set_codes.push({set_code: set.code})
    })

    const cardsFilter: orCardFilterProps[] = [{sanitized_name: commander_name, OR: set_codes}]
    cards.forEach((card:cardEntryAPIProps) => {
      cardsFilter.push({sanitized_name: card.sanitized_name, OR: set_codes})
    })

    const cardsData = await db.carte.findMany({
      where: {
        OR: cardsFilter
      }
    })
    
    let allCardFound = true
    if(cardsData.findIndex(cardData => cardData.sanitized_name == commander_name) == -1){
      allCardFound = false
    }
    cards.forEach((card: cardEntryAPIProps) => {
      if(cardsData.findIndex(cardData => cardData.sanitized_name == card.sanitized_name) == -1){
        console.log(card)
        allCardFound = false
      }
    })
    
    if(!allCardFound) {
      return NextResponse.json({"message": "Some cards were not found..."},{
        status: 401,
      });
    }

    const deck = await db.deck.create({
      data: {
        name,
        utilisateurice: {
          connect: {
            id: tokenData.id
          }
        },
        commander: {
          connect: {
            id: cardsData[cardsData.findIndex(cardData => cardData.sanitized_name == commander_name)].id
          }
        },
        bset: {
          connect: {
            id: bset?.id
          }
        }
      }
    })

    cards.forEach(async (card: cardEntryAPIProps) => {
      const cardData_id = cardsData[cardsData.findIndex(cardData => cardData.sanitized_name == card.sanitized_name)].id
      console.log(card.sanitized_name)
      await db.cartes_dans_deck.create({
        data: {
          amount: card.amount,
          carte: {
            connect: {
              id: cardData_id
            }
          },
          deck: {
            connect: {
              id: deck.id
            }
          }
        }
      })
    })

    return NextResponse.json({"data": deck, "message": "Deck created !"},{
      status: 200,
    });
  } catch (error) {
    console.log(error)
    return NextResponse.json(
      { error: "Failed, check console" },
      {
        status: 500,
      }
    );
  }
}
